/* This file is part of the Amalthea library.
 *
 * Copyright (C) 2018-2021 Eugene 'Vindex' Stulin
 *
 * The Amalthea library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This libary is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library. If not, see <http://www.gnu.org/licenses/>.
 */

module amalthea.libcore;

package import std.file;
package import std.conv;
package import std.exception;
package import std.stdio;
package import std.typecons;

import core.stdc.string : strerror_r;
import std.string : fromStringz;

/// Signed size_t.
alias ssize_t = ptrdiff_t;


/*******************************************************************************
 * Template for exceptions.
 */
mixin template RealizeException() {
    this(string msg, string file = __FILE__, size_t line = __LINE__) {
        super(msg, file, line);
    }
}


/*******************************************************************************
 * Returns string containing text view of an errno value.
 */
string getInfoAboutError(int err) {
    char[256] buffer;
    strerror_r(err, buffer.ptr, 256);
    return buffer.ptr.fromStringz.idup;
}

